﻿using ProjectAiroplane.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAiroplane.Drawnings;

public static class ExtentionDrawningPlane
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";
    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static Drawningplane? CreateDrawningplane(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        Entityplane? airoplane = EntityAiroplane.CreateEntityAiroplane(strs);
        if (airoplane != null)
        {
            return new DrawningAiroplane((EntityAiroplane)airoplane);
        }
        airoplane = Entityplane.CreateEntityplane(strs);
        if (airoplane != null)
        {
            return new Drawningplane(airoplane);
        }
        return null;
    }
    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningplane">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this Drawningplane drawningplane)
    {
        string[]? array = drawningplane?.Entityplane?.GetStringRepresentation();
        if (array == null)
        {
            return string.Empty;
        }
        return string.Join(_separatorForObject, array);
    }
}
