﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace ProjectAiroplane.Entities;

/// <summary>
/// Класс-сущность "Самолёт"
/// </summary>
public class Entityplane
{
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }
    /// <summary>
    /// Вес
    /// </summary>
    public double Weight { get; private set; }
    /// <summary>
    /// Основной цвет
    /// </summary>
    public Color BodyColor { get; private set; }
    /// <summary>
    /// Шаг перемещения самолета
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Основной цвет
    /// </summary>
    /// <param name="color"></param>
    public void setBodyColor(Color color)
    {
        BodyColor = color;
    }

    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed"></скорость>
    /// <param name="weight"></вес>
    /// <param name="bodyColor"></Основной цвет>

    public Entityplane(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(Entityplane), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static Entityplane? CreateEntityplane(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(Entityplane))
        {
            return null;
        }
        return new Entityplane(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}

