﻿using ProjectAiroplane.Drawnings;
using ProjectAiroplane.Entities;

namespace ProjectAiroplane;
/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormPlanConfig : Form
{
    /// <summary>
    /// Объект - прорисовка самолёта
    /// </summary>
    private Drawningplane _plane = null;

    private event Action<Drawningplane>? PlaneDelegate;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormPlanConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;
        // TODO buttonCancel.Click привязать анонимный метод через lambda закрытием формы
        buttonCancel.Click += (sender, e) => Close();
    }

    /// <summary>
    /// Привязка внешнего метода к событию вопрос 2..............................................................................
    /// </summary>
    public void AddEvent(Action<Drawningplane> planeDelegate)
    {
        PlaneDelegate += planeDelegate;
    }

    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _plane?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _plane?.SetPosition(5, 5);
        _plane?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }
    /// <summary>
    /// Действия при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _plane = new Drawningplane((int)numericUpDownSpeed.Value, (double)numericUpDownWeght.Value, Color.White);
                break;
            case "labelModifiedObject":
                _plane = new DrawningAiroplane((int)numericUpDownSpeed.Value, (double)numericUpDownWeght.Value, Color.White,
                    Color.Black, checkBoxToplivbak.Checked, checkBoxRadar.Checked);
                break;
        }
        labelBodyColor.BackColor = Color.Empty;
        labelAdditionalColor.BackColor = Color.Empty;
        DrawObject();
    }

    private void Panel_MouseDown(object sender, MouseEventArgs e)
    {
        // TODO отправка цвета в Drag&Drop
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }

    // TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)
    /// <summary>
    /// Проверка получаемой информации по основному цвету
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    /// <summary>
    /// действия при приеме перетаскиваемого основного цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane != null)
        {
            _plane.Entityplane.setBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }

    /// <summary>
    /// Передача объекта 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)// кнопку нажимаем и новый объект передается в форму 2 вопрос............
    {
        if (_plane != null)
        {
            PlaneDelegate?.Invoke(_plane); //Вызов события, с помощью инвок, 
            Close();
        }
    }

    /// <summary>
    /// действия при перетаскивании доп. цвета
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_plane.Entityplane is EntityAiroplane _airoplaneplane)
        {
            _airoplaneplane.setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();
    }

    private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_plane != null && _plane is DrawningAiroplane)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }
}
