﻿using ProjectAiroplane.CollectionGenericObjects;
using ProjectAiroplane.Drawnings;
using System.Windows.Forms;

namespace ProjectAiroplane;

public partial class FormPlaneCollection : Form
{
    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Хранилище коллеций
    /// </summary>
    private readonly StorageCollection<Drawningplane> _storageCollection;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormPlaneCollection()
    {
        InitializeComponent();
        _storageCollection = new();
    }
    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    ///
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }
    /// <summary>
    /// Добавление самолёта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddPlane_Click(object sender, EventArgs e)
    {
        FormPlanConfig form = new();
        // TODO передать метод

        form.Show();
        form.AddEvent(SetPlane);

    }

    /// <summary>
    /// Добавление самолёта в коллекцию
    /// </summary>
    /// <param name="excavator"></param>
    private void SetPlane(Drawningplane? plane)
    {
        if (_company == null || plane == null)
        {
            return;
        }

        if (_company + plane != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ButtonDelPlane_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBox1.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBox1.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }
    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        Drawningplane? plane = null;
        int counter = 100;
        while (plane == null)
        {
            plane = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }
        if (plane == null)
        {
            return;
        }
        FormAiroplane form = new()
        {
            Setplane = plane///
        };
        form.ShowDialog();
    }
    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonReFresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }
    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка",
            MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }
        _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        // TODO прописать логику удаления элемента из коллекции
        // нужно убедиться, что есть выбранная коллекция
        // спросить у пользователя через MessageBox, что он подтверждает, чтохочет удалить запись
        // удалить и обновить ListBox
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Обновление спсика в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    /// <summary>
    /// Создать компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        ICollectionGenericObjects<Drawningplane>? collection =
        _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new PlaneSharingService(pictureBox.Width, pictureBox.Height, collection);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();

    }

    private void saveToolStripMenuItem_Click_1(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_storageCollection.SaveData(saveFileDialog.FileName))
            {
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show("Не сохранилось", "Результат",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

    private void loadToolStripMenuItem_Click_1(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            if (_storageCollection.LoadData(openFileDialog.FileName))
            {
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                RerfreshListBoxItems();
            }
            else
            {
                MessageBox.Show("Не загрузилось", "Результат",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
